class WebTechnologies:
    def __init__(self, soup):
        dict = self.scrape_dict(soup)
        
    def _scrape_dict(self, soup):
        dict = {
                "gerald": "14",
                "neinold": "12"
                }



from bs4 import BeautifulSoup

class GeneralInformation:
    def __init__(self, soup):
        self.container = self._scrape_gi(soup)
        self.hostnames = self._scrape_hostnames(self.container)
        # self.domains = self._scrape_domains(self.containter) 
        # self.country = self._scrape_country(self.containter) 
        # self.organization = self._scrape_org(self.containter)
        # self.isp = self._scrape_isp(self.containter)
        # self.asn = self._scrape_asn(self.containter) 
        # self.last_seen = self._scrape_last_seen(self.containter)
        # self.tags = self._scrape_tags(self.containter)
    
    def _scrape_gi(self, soup):
        return soup.find('div', class_='card card-yellow card-padding')

    def _scrape_hostnames(self, container):
        print(container)
        



import click
import dns.resolver                     # dns_lookup
import socket
import re                               # for validate_domainname(target)
import requests                         # for get_html(ip)
from bs4 import BeautifulSoup

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By

from IPResult.ip_result import IpResult
from IPResult.general_information import GeneralInformation 
from IPResult.vulnerabilities import Vulnerabilities


# dns lookup
def dns_lookup(domainname):
    try:
        results = dns.resolver.resolve(domainname, 'A')
        counter = 1
        for val in results:
            print(f"Address record [{counter}]: ", val.to_text())
            counter += 1
    except dns.resolver.NoAnswer:
        print(f"Error: No A Records found for {domainname}.")
    except dns.resolver.NXDOMAIN:
        print(f"Error: Domain {domainname} does not exist.")
    except dns.exception.Timeout:
        print(f"Error: DNS resolution for \"{domainname}\" timed out.")
    except dns.resolver.ResolverError:
        print(f"Error: Resolver Error occurred for {domainname}.")


# validate ipv4-address input
def validate_ipv4(ip):
    try:
        socket.inet_pton(socket.AF_INET, ip)
        return True
    except socket.error:
        return False


# validate domain name input
def validate_domainname(domainname):
    domain_reg = r"^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\.)+[A-Za-z]{2,6}$"
    return bool(re.match(domain_reg, domainname))


# check for /host/<ip>, returns html upon success, raises error otherwise
def get_html(ip):
    url = f"https://www.shodan.io/host/{ip}"
    try:
        response = requests.get(url)
        # Raise error for responses other than 200
        response.raise_for_status()
        return response.content
    except requests.HTTPError as e:
        if e.response.status_code == 404:
            print("Page not found")
        elif e.response.status_code == 403:
            print("Forbidden access")
        else:
            print("Status code: ", e.response.status_code)
    except requests.RequestException as e:
        print("Error:", e)


# ipv4 handling for shodan
def get_ipv4_shodan(ipv4_addr):
    driver = webdriver.Chrome()
    driver.get(f"https://www.shodan.io/host/{ipv4_addr}")
    page_source = driver.page_source
    driver.quit()
    return page_source



# click cli
@click.command()
@click.argument('target')  # target is either a ipv4-address or a domainname
def main(target):
    print(target)
    if validate_domainname(target):
        click.echo(f"results for: {target}")
        dns_lookup(target)
        # TODO: handle domainname
        print("TODO: handle domainname")
    elif validate_ipv4(target):
        # TODO: handle ipv4

        page_source = get_ipv4_shodan(target)        
        soup = BeautifulSoup(page_source, 'html.parser')

        # IPResult
        ip_result = IpResult(soup)

        # General Information
        gen_inf = ip_result.general_information 

        # Open Ports
        # ports_div = soup.find('div', class_='card card-light-blue card-padding')

        # Web Technologies
        # web_techs = ip_result.web_technologies

        # Vulnerabilities
        vulns = ip_result.vulnerabilities 
        # print(vulns.table)

               # print(ports_div)

        # TODO: generate ip-result object
    else:
        click.echo("Invalid input")
        # TODO: more verbose error handling


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name='shox',
    version='0.1.0',
    description='A powerful tool that combines the port-scanning and OSINT-gathering capabilities of shodan.io with the onhand downloadable exploits of ExploitDB.',
    author='Vadin02',
    url='https://gitlab.com/JIbald/shox',
    packages=find_packages(),
    install_requires=[
        'flask',
        'flasgger',
        'click',
        'requests',
        'beautifulsoup4',
        'selenium',
    ],
    entry_points={
        'console_scripts': [
            'shox-api=api:main',
            'shox-cli=cli:call_api',
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)


import re
import socket
import dns.resolver
import requests
from bs4 import BeautifulSoup

""" from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys """


# dns lookup
def dns_lookup(domainname):
    try:
        results = dns.resolver.resolve(domainname, "A")
        counter = 1
        for val in results:
            print(f"Address record [{counter}]: ", val.to_text())
            counter += 1
    except dns.resolver.NoAnswer:
        print(f"Error: No A Records found for {domainname}.")
    except dns.resolver.NXDOMAIN:
        print(f"Error: Domain {domainname} does not exist.")
    except dns.exception.Timeout:
        print(f'Error: DNS resolution for "{domainname}" timed out.')
    except dns.resolver.ResolverError:
        print(f"Error: Resolver Error occurred for {domainname}.")


# validate ipv4-address input
def validate_ipv4(ip):
    try:
        socket.inet_pton(socket.AF_INET, ip)
        return True
    except socket.error:
        return False


# validate domain name input
def validate_domainname(domainname):
    domain_reg = r"^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\.)+[A-Za-z]{2,6}$"
    return bool(re.match(domain_reg, domainname))


# check for /host/<ip>, returns html upon success, raises error otherwise
def get_html(ip):
    url = f"https://www.shodan.io/host/{ip}"
    try:
        response = requests.get(url, headers={"User-Agent": "curl/7.0"})
        # Raise error for responses other than 200
        response.raise_for_status()
        return BeautifulSoup(response.text, "html.parser")
    except requests.HTTPError as e:
        if e.response.status_code == 404:
            print("Page not found")
        elif e.response.status_code == 403:
            print("Forbidden access")
        else:
            print("Status code: ", e.response.status_code)
    except requests.RequestException as e:
        print("Error:", e)


""" def get_ipv4_soup(ipv4_addr):
    driver = webdriver.Chrome()
    driver.get(f"https://www.shodan.io/host/{ipv4_addr}")
    page_source = driver.page_source
    driver.quit()
    return BeautifulSoup(page_source, 'html.parser') """



from .general_information import GeneralInformation
from .web_technologies import WebTechnologies
from .open_ports import OpenPorts
from .vulnerabilities import Vulnerability
from .vulnerabilities import Vulnerabilities 

class IpResult:
    def __init__(self, soup):
        self.general_information = GeneralInformation(soup) 
        # self.open_ports = OpenPorts(soup) 
        # self.web_technologies = WebTechnologies(soup) 
        self.vulnerabilities = Vulnerabilities(soup) 



from bs4 import BeautifulSoup

class Vulnerability:
    def __init__(self, cve_code, description, scoring='Unknown score'):
        self.cve_code = cve_code 
        self.description = description 
        self.scoring = scoring 

class Vulnerabilities:
    def __init__(self, soup):
        self.container = self._scrape_vulns_container(soup)
        self.table = self._scrape_vulns_table(self.container)
        self.arr_vulns = self._scrape_vulns(self.table)
        self.arr_cve_codes = self._get_cve_codes(self.arr_vulns)

    def _scrape_vulns_container(self, soup):
        return soup.find('div', class_='card card-red card-padding')

    def _scrape_vulns_table(self, container):
        try:
            table = container.find('table', id='vulnerabilities')
            if table:
                return table
            else:
                print("Table not found")
        except Exception as e:
            print("Exception: ", e)

    def _scrape_vulns(self, table):
        arr_vulns = []
        rows = table.find_all('tr')
        for row in rows:
            cells = row.find_all('td')
            if len(cells) == 2:
                cve_code = cells[0].text.strip()
                spans = cells[1].find_all('span')
                if len(spans) == 2:
                    scoring_span = cells[1].find('span', class_='tag')
                    description_span = cells[1].find('span', class_=False)
                    scoring = scoring_span.text.strip() if scoring_span else None
                    description = description_span.text.strip() if description_span else None
                else:
                    description = cells[1].text.strip()
                    scoring = "Unknown Scoring"

                vulnerability_obj = Vulnerability(cve_code, description, scoring)
                arr_vulns.append(vulnerability_obj)
        return arr_vulns

    def _get_cve_codes(self, arr_vulns):
        arr_cve_codes = []
        for vuln_obj in arr_vulns:
            arr_cve_codes.append(vuln_obj.cve_code)
        return arr_cve_codes

    # TODO: remove, this is for debugging
    def _print_arr_vulns(self, arr_vulns):
        count = 1
        for vuln_obj in arr_vulns:
            print("CVE Code: ", vuln_obj.cve_code)
            print("description: ", vuln_obj.description)
            print("scoring: ", vuln_obj.scoring)
            print("-" * 30, count)
            count += 1


    # TODO: remove, this is for debugging
    def _print_arr(self, arr):
        for entry in arr:
            print(entry)
            print("-" * 30)
